//
//	test the CcdQuantumDataServer class
//
#include <iostream>
#include <cassert>
#include "CalServer.h"
#include "Ccf.h"
#include "CcdQuantumDataServer.h"

using namespace std;

namespace {
bool isClose(double a, double b)
{
    return fabs(a)/fabs(b)-1. < 1.e-5;
}
}

int main()
{
	//	setup the the Cal server for MOS
	Xmm::Instrument instrument = Xmm::EMOS1;
	calServer.setInstrument(instrument);
	calServer.state()->ccd()->set(1);
	calServer.state()->node()->set(EMOScam::PRIMARY_NODE);
	calServer.state()->filter()->set(Xmm::THIN1);

	//	get the QE DataServer
	CcdQuantumDataServer *qds = 0; qds = calServer.getAtom(qds);

	double e[] = { 100., 1000., 3000., 5000., 9000. },
	f1v[] = { 0.0892135, 0.568432, 0.686166, 0.497647, 0.143326 }, 
	f2v[] = { 0.0553745, 0.570339, 0.686137, 0.49693, 0.14316 };

	const CcdQuantumEfficiencyCurve &ef = qds->at(300, 300).pattern(0),
									 eff = qds->at(300, 300).pattern(0);
	for (unsigned n=0; n<NUM_OF(e); ++n) {
		double ft = ef.efficiency(e[n]),
			   fi = ef.efficiency(e[n], e[n]+10.);
		cout<<"QE at E="<<e[n]<<"="<<ft
			<<" - integrated over 10 eV: "<<fi<<endl;
		assert(isClose(ft, f1v[n]) && isClose(fi, f2v[n]));
	}
	calServer.releaseAtom(qds);

        //    get the large to 13_25 pattern ratio
        /*
        qds = calServer.getAtom(qds);
        const double patrat = qds->largeTo13_25();

        cout<<"Pattern size ratio: "<<patrat<<endl;
        assert(patrat > 0.0);

	calServer.releaseAtom(qds);
        */

        //    get the pattern size array
        qds = calServer.getAtom(qds);
                                                                                
        double e2[] = { 100., 1000., 3000., 5000., 9000. };
                   //f3v[] = { 3.401, 0.943739, 0.980199, 0.790405, 0.250872 },
                   //f4v[] = { 0.0509488, 0.944441, 0.980239, 0.789496, 0.250521 };
        const CcdLargePatternCurve &ef2 = qds->sizeofLargePatterns();
        //const CcdLargePatternCurve &ef2 = qds->sizeofLargePatterns();
        for (unsigned n=0; n<NUM_OF(e2); ++n) {
                double ft2 = ef2.efficiency(e2[n]),
                           fi2 = ef2.efficiency(e2[n], e2[n]+10.);
                cout<<"Pattern size at E="<<e2[n]<<"="<<ft2
                        <<" - integrated over 10 eV: "<<fi2<<endl;
                assert( ft2 > 1.0 && fi2 > 1.0 );
        }
	calServer.releaseAtom(qds);

        // Get the large pattern to pattern 13-25 ratio from miscdata
        MiscDataServer * mds = 0; mds = calServer.getAtom(mds);
        if (mds->has("LARGE_TO_1325")) {
          double ratio = mds->get("LARGE_TO_1325");
          double expected_ratio = 8.34;
	  cout<<"Ratio of large patterns to 13-25: "<<ratio<<endl;
  	  assert(isClose(ratio, expected_ratio));
        }
	calServer.releaseAtom(mds);
                                                                     
	//	setup the the Cal server for RGS
	instrument = Xmm::RGS1;
	calServer.setInstrument(instrument);
	calServer.state()->ccd()->set(5);

	//	get the QE DataServer
	qds = calServer.getAtom(qds);

	double e3[] = { 100., 1000., 3000., 5000., 9000. },
		   f5v[] = { 0.0500342, 0.943739, 0.980199, 0.790405, 0.250872 },
		   f6v[] = { 0.0509488, 0.944441, 0.980239, 0.789496, 0.250521 };

    qds->ccdCorrFactors(1);
	const CcdQuantumEfficiencyCurve &ef3 = qds->grade(1);


	for (unsigned n=0; n<NUM_OF(e3); ++n) {
		double ft3 = ef3.efficiency(e3[n]),
			   fi3 = ef3.efficiency(e3[n], e3[n]+10.);
		cout<<"QE at E="<<e3[n]<<"="<<ft3
			<<" - integrated over 10 eV: "<<fi3<<endl;
		assert(isClose(ft3, f5v[n]) && isClose(fi3, f6v[n]));
	}

                                                                       
	return 0;
}
