//
//	test RGS pixel size in OCB 3x3 / ROWCOORDs; print ROWCOORD coordinates
//	of outer corners of first N (cmd line arg) pixel rows/columns
//
#include <iostream>
#include <iomanip>
#include <cmath>
#include <cstdlib>
#include "CalServer.h"
#include "Xmm.h"
#include "GeometryDataServer.h"
#include "common.h"

using namespace std;

namespace {
	bool isClose(double a, double b) {
		double d = fabs(fabs(a/b)-1.);
//		cout<<setprecision(12)<<a<<'/'<<b<<' '<<d<<endl;
		return d < 1.e-2;
	}
}

int main(int argc, char **argv)
{
	calServer.setInstrument(Xmm::RGS1);
	calServer.state()->randomize()->set(false);
	calServer.state()->ccd()->set(2);
	calServer.state()->ocb()->set(3);

	unsigned N = 5;
	if (argc == 2)
		N = atoi(argv[1]);

	CalReal64Vector	rawXVec(0, 5*N),
				    rawYVec(0, 5*N);

	double d = .5;
	for (unsigned n=0; n<N; ++n) {
		rawXVec[n*5] = DBL(n+1)-d;		rawYVec[n*5] = DBL(n+1)-d;
		rawXVec[n*5+1] = DBL(n+1)-d;	rawYVec[n*5+1] = DBL(128-n)+d;
		rawXVec[n*5+2] = DBL(342-n)+d;	rawYVec[n*5+2] = DBL(128-n)+d;
		rawXVec[n*5+3] = DBL(342-n)+d;	rawYVec[n*5+3] = DBL(n+1)-d;
		rawXVec[n*5+4] = DBL(n+1)-d;	rawYVec[n*5+4] = DBL(n+1)-d;
	}

#if 0
	for (unsigned n=0; n<N; ++n) {
		cerr<<rawXVec[n*5]<<'/'<<rawYVec[n*5]<<endl;
		cerr<<rawXVec[n*5+1]<<'/'<<rawYVec[n*5+1]<<endl;
		cerr<<rawXVec[n*5+2]<<'/'<<rawYVec[n*5+2]<<endl;
		cerr<<rawXVec[n*5+3]<<'/'<<rawYVec[n*5+3]<<endl;
		cerr<<rawXVec[n*5+4]<<'/'<<rawYVec[n*5+4]<<endl;
	}
#endif

	GeometryDataServer *cd = 0; cd = calServer.getAtom(cd);
	XmmRealPixelList pl = cd->chipCoordList(rawXVec, rawYVec);
	XmmCartesianPointList cpl = pl.toCamCoord1().toCamCoord2().toSacCoord();
	XmmRowlandPointList rpl = cpl.toRowCoord();

	for (unsigned n=0; n<5*N; ++n)
		cout<<setprecision(12)<<rpl.beta()[n]<<' '<<rpl.chi()[n]<<endl;

	if (N>=5) {
		double dx1 = rpl.beta()[0]-rpl.beta()[5],
			   dx2 = rpl.beta()[8]-rpl.beta()[3],
			   dxnom = rpl.beta()[5]-rpl.beta()[10],
			   dy1 = rpl.chi()[0]-rpl.chi()[5],
			   dy2 = rpl.chi()[5]-rpl.chi()[10];
		assert(isClose(dx1, dx2));
		assert(isClose(2./3.*dxnom, dx1));	//	outermost pixel columns
											//	in OCB 3x3 have 2/3 nominal
											//	extent
		assert(isClose(dy1, dy2));
	}

	return 0;
}
