//
//	test sky position -> CHIPCOORD conversion
//
#include <iostream>
#include "CalServer.h"
#include "Xmm.h"
#include "GeometryDataServer.h"
#include "common.h"

using namespace std;

int main(int argc, char **argv)
{
    if (argc == 2)
        //  assume the argument is a LINCOORD replacement
        calServer.ccf()->registerNewCifEntry(argv[1]);

	calServer.setInstrument(Xmm::EMOS1);

	//
	//	try CAMCOORD1 -> CHIPCOORD conversion
	//

	//	on CCD:     1   1   1    2     0     2   1   1     5    0       0
	double xmm[] ={ 0., 0., 0.,  12., 1000., 12., 0., 0., -12., 1000., 1000. },
		   ymm[] ={ 0., 0., 0., -24., 1000., -24., 0., 0., 24., 1000., 1000. },
		   zmm[] ={ 0., 0., 0.,  0., 0., 0., 0., 0., 0., 0., 0., };

	unsigned ccdnom[] = { 1, 1, 1, 2, 0, 2, 1, 1, 5, 0, 0 };
	int	xnom[] = { 300, 300, 300, 315, 0, 315, 300, 300, 296, 0, 0 },
		ynom[] = { 300, 300, 300, 297, 0, 297, 300, 300, 293, 0, 0 };

	CalReal64Vector x(xmm, NUM_OF(xmm)),
					y(ymm, NUM_OF(ymm)),
					z(zmm, NUM_OF(zmm));
	
	GeometryDataServer *gds = 0; gds = calServer.getAtom(gds);
	XmmCartesianPointList cl = gds->camCoord1List(x, y, z);
	vector<Ccd> ccds = cl.onCcd();
	XmmPixelList pl = cl.toChipCoord();
	for (unsigned n=0; n<cl.x().size(); ++n) {
		cout<<n<<' '<<ccds[n]<<' '<<pl.x()[n]<<' '<<pl.y()[n]<<endl;
		assert(ccds[n] == ccdnom[n]);
		assert(pl.x()[n] == xnom[n] && pl.y()[n] == ynom[n]);
	}


	//
	//	now try the whole chain from SKYCOORD->CHIPCOORD
	//
	Boresight *bs = 0; bs = calServer.getAtom(bs);
	bs->withSacCoordOrientation(SpacecraftAttitude(0., 0., 0.));

	RightAscension srcRa = RightAscension(10./3600.*DEG_TO_RAD); // 10"
	Declination srcDec = Declination(50./3600.*DEG_TO_RAD);		// 30"
	EquatorialDirection src(srcRa, srcDec);

	XmmTelescopePointList tpl = bs->projectOntoTelCoord(src);
	XmmCartesianPointList cpl = tpl.toCamCoord2().toCamCoord1();
	vector<Ccd> ccd = cpl.onCcd();
	XmmPixelList ccl = cpl.toChipCoord();
	cout<<src<<" lies ";
	if (ccd[0] == Xmm::CCD_NONE)
		cout<<"not an any CCD - either outside FOV or in chip gap";
	else
		cout<<"on CCD "<<ccd[0]<<" CHIPCOORD pixel: "
			<<ccl.x()[0]<<'/'<<ccl.y()[0];
	cout<<endl;
	assert(ccd[0] == 1);
	assert(ccl.x()[0] == 245 && ccl.y()[0] == 283);

	return 0;
}
